/*
	File: CCONNMGR.H
	Description:
		A stream socket connection manager class.
	Copyright 1995, David G. Roberts
*/

#ifndef _CCONNMGR_H
#define _CCONNMGR_H

#ifndef _CSOCKET_H
#include "csocket.h"
#endif
#ifndef _CHOST_H
#include "chost.h"
#endif
#ifndef _CSOCKADR_H
#include "csockadr.h"
#endif

class CConnMgrCallback {
public:
	enum RESULT {
		INVALID_RESULT,
		ABORTED,
		OK_CONNECTED,
        ERR_NOTCONNECTED
	};
	virtual void ConnMgrCallback(int iToken, RESULT result) = 0;
};

class CConnMgr : private CAsyncDBCallback, private CStreamCallback {
private:
	enum EVENT { NULLEVENT, LOOKUPDONE, CONNECT, ABORT };

	typedef BOOL (CConnMgr::*PSTATEFUNC)(EVENT ev);

	CConnMgrCallback * m_pManagerCallback;
	int			m_iManagerToken;
	CConnMgrCallback::RESULT m_result;
	char *		m_pszHost;
	u_short		m_ushPort;
	CStreamSocket * m_pSocket;
	CStreamCallback * m_pStreamCallback;
	int			m_iStreamToken;
	PSTATEFUNC	m_pState;
    BOOL		m_bDotted;
	CHost		m_host;
	CSockAddr	m_saAddr;
	int			m_iError;
	BOOL		m_bDoCallback;

    // State machine interface.
	void Event(EVENT ev);
	void NextState(PSTATEFUNC pState) { m_pState = pState; };

	// State functions.  Return value is TRUE if they responded to the
	// event.  If the event is ignored, the state function should return
    // FALSE.
	BOOL Start(EVENT ev);
	BOOL AwaitLookup(EVENT ev);
	BOOL MakeConnection(EVENT ev);
	BOOL AwaitConnection(EVENT ev);
	BOOL Exit(EVENT ev);
	BOOL Stop(EVENT ev);

	// Callback handlers for stream sockets and host lookups.
	virtual void AsyncDBCallback(int iToken, int iError);
	virtual void StreamCallback(int iToken, STREAMEVENT iEvent, int iError,
		char FAR * lpBuffer);

	// Eliminate the use of the copy constructor and operator=.
	CConnMgr(const CConnMgr&);
    operator=(const CConnMgr&);
public:
	CConnMgr(void) : m_pManagerCallback(NULL), m_iManagerToken(0),
			m_pState(&CConnMgr::Stop) {};
	virtual ~CConnMgr() {};

	void SetCallback(CConnMgrCallback * pCallback, int iToken)
		{ m_pManagerCallback = pCallback; m_iManagerToken = iToken; };
	BOOL Connect(LPCSTR lpszHost, u_short ushPort, CStreamSocket * pSocket,
		CStreamCallback * pCallback, int iToken);
	void Abort(BOOL bDoCallback = FALSE);

    int LastError(void) { return m_iError; };
};

#endif
