/*
	File: CFINGOBJ.H
	Description:
		An class to implement the finger protocol.
	Copyright 1995, David G. Roberts
*/

#ifndef _CFINGOBJ_H
#define _CFINGOBJ_H

#include <windows.h>
#include "winsock.h"
#ifndef _CSOCKADR_H
#include "csockadr.h"
#endif
#ifndef _CHOST_H
#include "chost.h"
#endif
#ifndef _CSOCKET_H
#include "csocket.h"
#endif

/*
	Class: CFingerObj
	Description:
		Implements the finger protocol.  Sends a program-defined
		message to a window when the protocol is complete along with
		the lines of text returned from the remote host.
*/
class CFingerObj : private CAsyncDBCallback, private CStreamCallback {
	enum STATE { START, AWAITLOOKUP, MAKECONNECTION, AWAITCONNECTION,
		CONNECTED, AWAITDATA, CLOSED, ERROREXIT, FINISH };
	enum EVENT { NULLEVENT, LOOKUPDONE, CLOSE, CONNECT, DATAREADY,
		DATASENT };
	enum CONSTANTS { BUFFER_QUANTUM = 128 };

	struct LineNode {
		LineNode *	m_plnNext;
		char *		m_pszLine;

		LineNode() : m_plnNext(NULL), m_pszLine(NULL) {};
	};

	char *		m_pszHost;
	u_short		m_ushPort;
	char *		m_pszUser;
	HWND		m_hwnd;
	int			m_iMessage;
	HINSTANCE	m_hInstance;
	STATE		m_stState;
	EVENT		m_evEvent;
	int			m_iError;
	char FAR *	m_lpSentBuffer;
	CSockAddr	m_saAddr;
    BOOL		m_bDotted;
	CHost		m_host;
	CStreamSocket m_stream;
	LineNode	m_lnHead;
	LineNode *	m_plnTail;
	char *		m_pDataBuffer;
	unsigned	m_uDataSize;
	unsigned	m_uBufferSize;

	// Event dispatch function.
	void Event(EVENT ev);
	// Functions for each state.
	BOOL Start(EVENT ev);
	BOOL AwaitLookup(EVENT ev);
	BOOL MakeConnection(EVENT ev);
	BOOL AwaitConnection(EVENT ev);
	BOOL Connected(EVENT ev);
	BOOL AwaitData(EVENT ev);
	BOOL Closed(EVENT ev);
	BOOL ErrorExit(EVENT ev);

	// Helpful error reporting function.
	void ErrorWithHostName(LPCSTR lpszTitle, LPCSTR lpszErrorMsg);

	// Callbacks.
	virtual void AsyncDBCallback(int iToken, int iError);
	virtual void StreamCallback(int iToken, STREAMEVENT iEvent,
		int iError, char FAR * lpBuffer);
public:
	CFingerObj(LPCSTR lpszHost, u_short ushPort, LPCSTR lpszUser,
		HWND hwnd, int iMessage, HINSTANCE hInstance);
	~CFingerObj();
};

#endif
