/*
	File: CHOST.CPP
	Description:
		A class to perform host name lookup.
	Copyright 1995, David G. Roberts
*/

#include "chost.h"
#include "myassert.h"

/*
	Function: StartWinsockLookup
	Description:
		Virtual function provided to CAsyncDB to kick off the right
		type of lookup.
*/
HANDLE CHost::StartWinsockLookup(void)
{
	return WSAAsyncGetHostByName(GetEventWindow(),
		WM_WSAGETHOSTBYNAME, m_lpszHostName, (char FAR *) m_pBuffer,
		MAXGETHOSTSTRUCT);
}

/*
	Function: CHost::FirstAddr
	Description:
		Retrieves the first host address following a lookup.
	Inputs:
		pbResult -- pointer to a BOOL indicating whether the
		operation was successful (it won't be successful if the
		CHost state is invalid or if the looked up host doesn't have
		even one address for some reason).
	Outputs:
		Returns a CSockAddr.
*/
CSockAddr CHost::FirstAddr(BOOL * pbResult)
{
	ASSERT(pbResult != NULL);

	// If we're still looking something up, indicate failure.
	if (m_stState != VALID) {
		*pbResult = FALSE;
		return CSockAddr(0, 0);
	}
	m_iAddrIndex = 0;		// Move us to the first index.
	// Make sure this entry is valid.
	if (((HOSTENT *)m_pBuffer)->h_addr_list[m_iAddrIndex] == NULL) {
		*pbResult = FALSE;
		return CSockAddr(0, 0);
	}
	// Get it.
	CSockAddr saResult(0, ntohl(*(u_long FAR *)
		(((HOSTENT *)m_pBuffer)->h_addr_list[m_iAddrIndex])));
	m_iAddrIndex++;			// Move us to the next index.
	*pbResult = TRUE;
	return saResult;
}

/*
	Function: CHost::NextAddr
	Description:
		Retrieves the next host address following a lookup.
	Inputs:
		pbResult -- pointer to a BOOL indicating whether the
		operation was successful (it won't be successful if the
		CHost state is invalid or if there are no more addresses
		for the host).
	Outputs:
		Returns a CSockAddr.
*/
CSockAddr CHost::NextAddr(BOOL * pbResult)
{
	ASSERT(pbResult != NULL);

	// If we're still looking something up, indicate failure.
	if (m_stState != VALID) {
		*pbResult = FALSE;
		return CSockAddr(0, 0);
	}
	// Make sure this entry is valid.
	if (((HOSTENT *)m_pBuffer)->h_addr_list[m_iAddrIndex] == NULL) {
		*pbResult = FALSE;
		return CSockAddr(0, 0);
	}
	// Get it.
	CSockAddr saResult(0, ntohl(*(u_long FAR *)
		(((HOSTENT *)m_pBuffer)->h_addr_list[m_iAddrIndex])));
	m_iAddrIndex++;			// Move us to the next index.
	*pbResult = TRUE;
	return saResult;
}

