/*
	File: CNNTP.H
	Description:
		A class for interacting with an NNTP server.
	Copyright 1995, David G. Roberts
*/

#ifndef _CNNTP_H
#define _CNNTP_H

#ifndef _CSOCKET_H
#include "csocket.h"
#endif
#ifndef _CCONNMGR_H
#include "cconnmgr.h"
#endif
#ifndef _CSTRLIST_H
#include "cstrlist.h"
#endif
#include <fstream.h>

class CNNTP : private CConnMgrCallback, private CStreamCallback {
public:
	enum RESULT { RESULT_ABORT, RESULT_SUCCESS, RESULT_FAILURE,
		RESULT_FATALERROR };
	struct ARTICLEANDFILENAME {
		long	lArticleNum;
		char *	pszFilename;
	};
private:
	enum CONSTANTS { MAX_REPLY_SIZE = 1001, MAX_LINE_SIZE = 200 };
	enum EVENT { EV_NULLEVENT, EV_CONNECT, EV_CLOSE, EV_DATA,
		EV_ENDDATA, EV_CODE1, EV_CODE2, EV_CODE3, EV_CODE4,
		EV_CODE5, EV_ERROR, EV_ABORT, EV_GROUP, EV_NEXTHEADER,
		EV_HEADER, EV_ARTICLE, EV_QUIT};
	typedef BOOL (CNNTP::*PSTATEFUNC)(EVENT ev);

	RESULT			m_result;
	CConnMgr		m_connmgr;
	CStreamSocket	m_sock;
	char			m_szReceiveBuffer[MAX_REPLY_SIZE];
	int				m_iReceiveBufferLen;
	char			m_szReplyLineBuffer[MAX_REPLY_SIZE];
	char			m_szLineBuffer[MAX_LINE_SIZE];
	PSTATEFUNC		m_pState;
	int				m_iError;
	char *			m_pszHost;
	u_short			m_ushPort;
	char			m_szFileName[255];
	BOOL			m_bDoCallback;
	ofstream		m_ofsFile;
	HWND			m_hwnd;
	UINT			m_msg;
	BOOL			m_bDataMode;
	char *			m_pszGroup;
    long			m_lArticle;

	// State machine interface.
	void Event(EVENT ev);
	void NextState(PSTATEFUNC pState) { m_pState = pState; };

	// State functions.
	BOOL Start(EVENT ev);
	BOOL AwaitConnection(EVENT ev);
	BOOL AwaitGreeting(EVENT ev);
	BOOL Idle(EVENT ev);
	BOOL SendGroup(EVENT ev);
	BOOL AwaitGroupResponse(EVENT ev);
	BOOL SendNext(EVENT ev);
    BOOL AwaitNextResponse(EVENT ev);
	BOOL SendHeader(EVENT ev);
	BOOL OpenFile(EVENT ev);
	BOOL RetrieveLine(EVENT ev);
	BOOL SendArticle(EVENT ev);
	BOOL Quit(EVENT ev);
    BOOL AwaitQuitResponse(EVENT ev);
	BOOL Exit(EVENT ev);
	BOOL Stop(EVENT ev);

	// Aux functions.
	BOOL ReadReplyData(void);
	BOOL LineComplete(void);
	void CopyLine(void);
	void ShrinkReceiveBuffer(void);
    BOOL MessageTerminator(void);

	virtual void ConnMgrCallback(int iToken,
		CConnMgrCallback::RESULT result);
	virtual void StreamCallback(int iToken, STREAMEVENT iEvent,
		int iError, char FAR * lpBuffer);

	// Eliminate the use of the copy constructor and operator=.
	CNNTP(const CNNTP&);
	operator=(const CNNTP&);
public:
	CNNTP() : m_pszHost(NULL), m_pszGroup(NULL)
		{ m_connmgr.SetCallback(this, 0); };
	virtual ~CNNTP() { Abort(NULL, 0, FALSE); };

	void Connect(HWND hwnd, UINT msg, char * pszHost,
		u_short ushPort);
	void Group(HWND hwnd, UINT msg, char * pszGroup);
    void NextHeader(HWND hwnd, UINT msg);
	void Header(HWND hwnd, UINT msg);
	void Article(HWND hwnd, UINT msg, long lArticle);
	void Quit(HWND hwnd, UINT msg);

	void Abort(HWND hwnd, UINT msg, BOOL bDoCallback);

    int LastError(void) { return m_iError; };
};

#endif
