/*
	File: CSOCKADR.H
	Description:
		An Internet family socket address class.
	Copyright 1995, David G. Roberts
*/

#ifndef _CSOCKADR_H
#define _CSOCKADR_H

#include <windows.h>
#include "winsock.h"

/*
	Class: CSockAddr
	Description:
		The CSockAddr class is an encapsulation of a Winsock
		sockaddr/sockaddr_in structure.  The class eliminates the
		common error of forgetting to specify address or port numbers
		in host byte order.  If used with other objects that generate
		addresses in given formats compatible with this object, an
		application need not know of Internet address formats and thus
		may not have to change when converted to support IPv6.

		Note that all address and port numbers given and returned
		as integers are in host byte order.  The functions convert
        them to network byte order as appropriate.
*/
class CSockAddr {
private:
	SOCKADDR_IN		m_sinAddr;	// The address itself.
public:
	// Constructors and destructors
	CSockAddr() { m_sinAddr.sin_family = AF_INET; }; // Default
	CSockAddr(const CSockAddr& csaSockAddr)	// Copy constructor
		: m_sinAddr(csaSockAddr.m_sinAddr) {};
	CSockAddr(const SOCKADDR& saSockAddr)
		: m_sinAddr(*(SOCKADDR_IN FAR *)&saSockAddr) {};
	CSockAddr(const SOCKADDR_IN& sinSockAddr)
		: m_sinAddr(sinSockAddr) {};
	CSockAddr(const u_short ushPort, const u_long ulAddr)
		{ Set(ushPort, ulAddr); };
	~CSockAddr() {};	// Destructor
	// Overloaded operator= functions.
	const CSockAddr& operator=(const CSockAddr& csaSockAddr);
	const CSockAddr& operator=(const SOCKADDR& saSockAddr)
		{ m_sinAddr = *(SOCKADDR_IN FAR *)&saSockAddr;
		  return *this; };
	const CSockAddr& operator=(const SOCKADDR_IN& sinSockAddr)
		{ m_sinAddr = sinSockAddr; return *this; };
	// Functions for manipulating portions of SOCKADDR_IN after
	// construction time.
	void Set(const u_short ushPort, const u_long ulAddr);
	u_short Port(void) const
		{ return ntohs(m_sinAddr.sin_port); };
	void Port(const u_short ushPort)
		{ m_sinAddr.sin_port = htons(ushPort); };
	u_long Addr(void) const
		{ return ntohl(m_sinAddr.sin_addr.s_addr); };
	void Addr(const u_long ulAddr)
		{ m_sinAddr.sin_addr.s_addr = htonl(ulAddr); };
	// Conversion operators so we can use CSockAddr with Winsock
	// functions.
	operator SOCKADDR()
		{ return *((SOCKADDR *) &m_sinAddr); };
	operator SOCKADDR_IN()
		{ return m_sinAddr; };
	operator LPSOCKADDR()
		{ return (LPSOCKADDR) & m_sinAddr; };
	operator LPSOCKADDR_IN()
		{ return (LPSOCKADDR_IN) & m_sinAddr; };
};

#endif // #ifndef _CSOCKADR_H
