/*
	File: CSOCKET.H
	Description:
		Class to encapsulate a Windows socket.
	Copyright 1995, David G. Roberts
*/

#ifndef _CSOCKET_H
#define _CSOCKET_H

#include <windows.h>
#include "winsock.h"
#ifndef _CWINSOCK_H
#include "cwinsock.h"
#endif
#ifndef _CSOCKADR_H
#include "csockadr.h"
#endif

class CStreamCallback {
public:
	enum STREAMEVENT { SE_CONNECT, SE_ACCEPT, SE_DATAREADY,
		SE_DATASENT, SE_CLOSE };
	virtual void StreamCallback(int iToken, STREAMEVENT iEvent,
		int iError, char FAR * lpBuffer) = 0;
};

class CStreamSocket : private CWinsockEventCallback {
private:
	// BufferDescriptor used to chain buffered transmit data.
	struct BufferDescriptor {
		BufferDescriptor *	m_pbdNext;
		char FAR *			m_lpOriginalBuffer;
		char FAR *			m_lpCurrentBuffer;
		int					m_iLength;

		BufferDescriptor(char FAR * lpOriginalBuffer,
			char FAR * lpCurrentBuffer, int iLength)
			: m_pbdNext(NULL), m_lpOriginalBuffer(lpOriginalBuffer),
			m_lpCurrentBuffer(lpCurrentBuffer), m_iLength(iLength) {};
	};

	// State that a socket goes through.
	enum SOCKETSTATE { IDLE, CONNECTING, CONNECTED, LISTENING };

	SOCKET				m_s;
	int					m_iLastError;
	CStreamCallback *	m_pCallback;
	int					m_iToken;
	SOCKETSTATE			m_iState;
	BufferDescriptor *	m_pbdHead;
	BufferDescriptor *	m_pbdTail;

	virtual void WinsockEventCallback(HWND, UINT, WPARAM, LPARAM);

	// Private constructor used during AcceptConnection.
	CStreamSocket(SOCKET, CStreamCallback *, int);

	// Private functions to manage the send queue.
	void AddTail(char FAR *, char FAR*, int);
	void DeleteHead(void);

	// Make these off limits so nobody uses them.
	CStreamSocket(CStreamSocket&);			  // Copy constructor.
	CStreamSocket& operator=(CStreamSocket&); // Assignment operator.
public:
	// Constructor and destructor.
	CStreamSocket(void)
		: m_s(INVALID_SOCKET), m_iLastError(WSANOERROR),
		m_pCallback(NULL), m_iToken(0), m_iState(IDLE),
		m_pbdHead(NULL), m_pbdTail(NULL) {};
	virtual ~CStreamSocket();
    // CStreamSocket managment functions.
	BOOL DataQueued(void)
		{ return (m_pbdHead != NULL) ? TRUE : FALSE; };
	int LastError() { return m_iLastError; };
	void SetCallback(CStreamCallback * pCallback, int iToken)
    	{ m_pCallback = pCallback; m_iToken = iToken; };
	// General socket functions.
	CStreamSocket * Accept(CStreamCallback * pCallback, int iToken, 
		CSockAddr * pcsaAddr = NULL);
	int Bind(CSockAddr& saBindAddr);
	int Close(void);
	int Connect(CSockAddr& saForeignAddr);
	int GetPeerName(CSockAddr * psaPeerName);
	int GetSockName(CSockAddr * psaSockName);
	int GetSockOpt(int iLevel, int iOptName, char * pOptVal,
		int * pnLen);
	int IoctlSocket(long lCommand, u_long * pArg);
	int Listen(CSockAddr& saServiceAddr, int iQueueLen);
	int Open(void);
	int Receive(char FAR * lpBuffer, int * piBuffLength,
		int iFlags = 0);
	int Send(char FAR * lpBuffer, int iLength, int iFlags = 0);
	int SetSockOpt(int iLevel, int iOptName, char * pOptVal,
		int nLen);
};

#endif
