/*
	File: CSTRLIST.CPP
	Description:
		A class to store a list of strings.
	Copyright 1995, David G. Roberts
*/

#include <windows.h>
#include "cstrlist.h"
#include "myassert.h"

/*
	Function: CStringList::Compare
	Description:
		Compares two strings and return -1 if string1 < string2,
		0 if string1 == string2, and 1 if string1 > string2.
*/
int CStringList::Compare(void * pvThing1, void * pvThing2)
{
	return lstrcmp((char *) pvThing1, (char *) pvThing2);
}

/*
	Function: CStringList::DeleteData
	Description:
		Deletes the string data before a list node is destroyed.
*/
void CStringList::DeleteData(void * pvThing)
{
	delete[] (char *) pvThing;
}

/*
	Function: CStringList::Add
	Description:
		Add a string to the list.  The string passed as the
		function argument is copied and then added to the list.
*/
void CStringList::Add(char * pszString)
{
	ASSERT(pszString != NULL);
	char * pszStringCopy = new char[lstrlen(pszString) + 1];
	lstrcpy(pszStringCopy, pszString);
	CList::Add((void *) pszStringCopy);
}

/*
	Function: CStringList::Insert
	Description:
		Inserts a string into the list.  The string passed as the
		function argument is copied and then added to the list.
*/
void CStringList::Insert(char * pszString)
{
	ASSERT(pszString != NULL);
	char * pszStringCopy = new char[lstrlen(pszString)];
	lstrcpy(pszStringCopy, pszString);
	CList::Insert((void *) pszStringCopy);
}

/*
	Function: CStringList::~CStringList
	Description:
		Class destructor.  Deletes all the string objects before
		the list itself is deleted.  The function simply iterates
		over the list elements and calls delete[] for each.
*/
CStringList::~CStringList()
{
	char * pszString = First();
	while (pszString != NULL) {
		delete[] pszString;
		pszString = Next();
	}
}
