/*
	File: CPOP3.H
	Description:
		A class for retrieving email messages via POP3 (RFC 1460).
	Copyright 1995, David G. Roberts

    4/24/96: Corrected SMTP/POP3 typo in CPOP3Callback.
*/

#ifndef _CPOP3_H
#define _CPOP3_H

#ifndef _CSOCKET_H
#include "csocket.h"
#endif
#ifndef _CCONNMGR_H
#include "cconnmgr.h"
#endif
#ifndef _CSTRLIST_H
#include "cstrlist.h"
#endif
#include <fstream.h>

class CPOP3 : private CConnMgrCallback, private CStreamCallback {
public:
	enum RESULT { RESULT_ABORT, RESULT_SUCCESS, RESULT_FAILURE,
		RESULT_NOMAIL };
protected:
	RESULT			m_result;
private:
	enum CONSTANTS { MAX_REPLY_SIZE = 1001, MAX_LINE_SIZE = 200 };
	enum EVENT { EV_NULLEVENT, EV_CONNECT, EV_CLOSE, EV_DATAREADY,
		EV_ERROR, EV_ABORT};
	typedef BOOL (CPOP3::*PSTATEFUNC)(EVENT ev);

	CConnMgr		m_connmgr;
	CStreamSocket	m_sock;
	char			m_szReceiveBuffer[MAX_REPLY_SIZE];
	int				m_iReceiveBufferLen;
	char			m_szReplyLineBuffer[MAX_REPLY_SIZE];
	char			m_szLineBuffer[MAX_LINE_SIZE];
	PSTATEFUNC		m_pState;
	int				m_iError;
	char *			m_pszHost;
	u_short			m_ushPort;
	char *			m_pszUser;
	char *			m_pszPassword;
	char *			m_pszPathName;
	CStringList	*	m_pslFileList;
	char			m_szFileName[255];
	int				m_iTotalMessages;
	int				m_iRetrieved;
	BOOL			m_bDoCallback;
	ofstream		m_ofsFile;

	// State machine interface.
	void Event(EVENT ev);
	void NextState(PSTATEFUNC pState) { m_pState = pState; };

	// State functions.
	BOOL Start(EVENT ev);
	BOOL AwaitConnection(EVENT ev);
	BOOL AwaitGreeting(EVENT ev);
	BOOL AwaitUserResponse(EVENT ev);
	BOOL AwaitPassResponse(EVENT ev);
	BOOL AwaitStatResponse(EVENT ev);
	BOOL StartRetrieve(EVENT ev);
	BOOL AwaitRetrResponse(EVENT ev);
	BOOL RetrieveLine(EVENT ev);
    BOOL AwaitDeleResponse(EVENT ev);
	BOOL Quit(EVENT ev);
    BOOL AwaitQuitResponse(EVENT ev);
	BOOL Exit(EVENT ev);
	BOOL Stop(EVENT ev);

	// Virtual function called when the CPOP3 state machine has
	// finished.
	virtual void DoCallback(void) = 0;

	// Aux functions.
	BOOL ReadReplyData(void);
	BOOL LineComplete(void);
	void CopyLine(void);
	void ShrinkReceiveBuffer(void);
	BOOL PositiveResponse(void);
    BOOL MessageTerminator(void);
	void NewFileName(char * pszPathName, char * pszFileName,
		char * pszFullName);

	virtual void ConnMgrCallback(int iToken,
		CConnMgrCallback::RESULT result);
	virtual void StreamCallback(int iToken, STREAMEVENT iEvent,
		int iError, char FAR * lpBuffer);

	// Eliminate the use of the copy constructor and operator=.
	CPOP3(const CPOP3&);
	operator=(const CPOP3&);
public:
	CPOP3() { m_connmgr.SetCallback(this, 0); };
	virtual ~CPOP3() { Abort(FALSE); };

	BOOL RetrieveMessages(LPCSTR lpszHost, u_short ushPort,
		LPCSTR lpszUser, LPCSTR lpszPassword, LPCSTR lpszPathName,
		CStringList * pslFileList);
	void Abort(BOOL bDoCallback);
};

class CPOP3WithWinMsg : public CPOP3 {
private:
	HWND m_hwnd;
	UINT m_msg;
	int  m_iToken;

	virtual void DoCallback(void);
public:
	CPOP3WithWinMsg(HWND hwnd, UINT msg, int iToken) :
		m_hwnd(hwnd), m_msg(msg), m_iToken(iToken) {};
	~CPOP3WithWinMsg() {};
};

class CPOP3Callback {
public:
	virtual void POP3Callback(int iToken, CPOP3::RESULT result) = 0;
};

class CPOP3WithCallback : public CPOP3 {
private:
	CPOP3Callback * m_pCallback;
	int  m_iToken;

	virtual void DoCallback(void);
public:
	CPOP3WithCallback(CPOP3Callback * pCallback, int iToken) :
		m_pCallback(pCallback), m_iToken(iToken) {};
	~CPOP3WithCallback() {};
};
#endif
