/*
File: tracer.h
Description: Header file for general purpose tracing object.
History:
	March 1997, Dave Roberts, First written

Copyright 1997, David G. Roberts
*/

#ifndef TRACER_H
#define TRACER_H

#include <windows.h>
#include <iostream.h>
#include <strstrea.h>

// This is a singleton class.
class Tracer {
public:
	enum { TR_NONE = 0, TR_ERROR = 1, TR_RETVAL = 2, TR_ENTRY = 4,
		TR_EXIT = 8, TR_MSG = 16, TR_DETAILEDMSG = 32, 
		TR_ALL = 32+16+8+4+2+1 };
	enum { TR_LEVELINDENT = 2 };
	static Tracer * Instance(void);
	unsigned SetFlags(unsigned flags);
	unsigned ClearFlags(unsigned flags);
	unsigned AssignFlags(unsigned flags);
	unsigned GetFlags(void) const;
	int GetLevel(void) const;
	int SetLevel(int level);
	int IncrementLevel(void);
	int DecrementLevel(void);
protected:
	Tracer();
private:
	static Tracer * m_instance;

	unsigned	m_flags;
	int			m_level;
};

#ifdef NDEBUG
#define DEFAULT_FLAGS (Tracer::TR_NONE)
#else
#define DEFAULT_FLAGS (Tracer::TR_ERROR | Tracer::TR_RETVAL)
#endif

#define TRACEOUT(flags, x) \
{\
	if (Tracer::Instance()->GetFlags() & flags) { \
		ostrstream s;\
		for (int i = 0; i < (Tracer::Instance()->GetLevel() * int(Tracer::TR_LEVELINDENT)); i++)\
			s << ' ';\
		s << x << " : " << __FILE__ << '(' << __LINE__ << ')' << endl << ends;\
		OutputDebugString(s.str());\
	}\
}

class TracerEntryExit {
public:
	TracerEntryExit(const char * name);
	~TracerEntryExit();
private:
	const char *	m_name;
};

inline TracerEntryExit::TracerEntryExit(const char * name) : m_name(name)
{
	TRACEOUT(Tracer::TR_ENTRY, "Entering " << m_name);
	Tracer::Instance()->IncrementLevel();
}

inline TracerEntryExit::~TracerEntryExit()
{
	Tracer::Instance()->DecrementLevel();
	TRACEOUT(Tracer::TR_EXIT, "Exiting " << m_name);
}

#endif // #ifndef TRACER_H