/*
	File: FINGER.CPP
	Description:
		A simple client that implements the Finger protocol.
	Copyright 1995, David G. Roberts
*/

#include <windows.h>
#include <windowsx.h>
#include "ctl3d.h"
#include "winsock.h"
#include "cfingobj.h"
#include "finger.rh"
#include "myassert.h"

/*
	CONSTANTS
*/
#define DEFAULT_FINGER_PORT	79
#define WM_FINGERDONE		WM_USER

/*
	MACROS
*/
#define min(a, b) (((a) < (b)) ? (a) : (b))
#define max(a, b) (((a) > (b)) ? (a) : (b))
// Microsoft Visual C++ 1.5 WINDOWSX.H doesn't include this macro,
// so define it
#ifndef GET_WM_COMMAND_ID
#ifdef WIN32
#define GET_WM_COMMAND_ID(wp, lp)	LOWORD(wp)
#else
#define GET_WM_COMMAND_ID(wp, lp)	(wp)
#endif
#endif

/*
	TYPES
*/
typedef int INT;
typedef BOOL FAR * LPBOOL;

/*
	GLOBAL VARIABLES
*/
#ifdef WIN32
#define WINVERSION "32"
#else
#define WINVERSION "16"
#endif
char		g_szAppName[] = "FINGER";
char		g_szFullAppName[] = "Finger Client (Win" WINVERSION ")";
HWND		g_hFrame;
HINSTANCE	g_hInstance;
char *		g_pszHost;
u_short		g_ushPort;
u_short		g_ushDefaultPort;
char *		g_pszUser;
CWinsock	g_wsWinsock(0x0101, 0x0101);
char **		g_ppszFingerLines = NULL;
int			g_iNumLines = 0;
int			g_iLineHeight;
int			g_iClientHeight;
CFingerObj*	g_pFingerObj = NULL;

/*
	FUNCTION PROTOTYPES
*/
void		CleanupInstance(HINSTANCE);
BOOL CALLBACK ConnectDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK FrameWndProc(HWND, UINT, WPARAM, LPARAM);
BOOL		InitWinsock(HWND, HINSTANCE, HINSTANCE);
BOOL		InitApp(HINSTANCE);
BOOL		InitInstance(HINSTANCE);
void		InitError(void);
INT WINAPI	WinMain(HANDLE, HANDLE, LPSTR, INT);

/*
	MESSAGE HANDLER PROTOTYPES
*/
void	OnCommand(HWND, int, HWND, UINT);
void	OnFingerDone(HWND, WPARAM, LPARAM);
void	OnKey(HWND, UINT, BOOL, int, UINT);
void	OnPaint(HWND);
BOOL	OnSetCursor(HWND, HWND, UINT, UINT);
void	OnSize(HWND, UINT, int, int);
void	OnVScroll(HWND, HWND, UINT, int);

/*
	WM_COMMAND HANDLER PROTOTYPES
*/
void	OnFileExit(HWND);
void	OnHelpAbout(HWND);
void	OnNetworkConnect(HWND);

/*********************************************************************
	WINMAIN
*********************************************************************/

/*
	Function: WinMain
	Description:
		Good ol' WinMain.
*/
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpszCmdParam, INT nCmdShow)
{
	UNUSED_PARAMETER(lpszCmdParam);

	// Save our instance handle for later.
	g_hInstance = hInstance;

	// If no other instance is running, initialize the application
	// window classes.
	if (!hPrevInstance) {
		if (!InitApp(hInstance)) {
			InitError();
			return 0;
		}
	}

	// Initialize all instance-specific stuff.  In particular, create
    // the frame window.
	if (!InitInstance(hInstance)) {
		InitError();
		return 0;
	}

	// Since initializing Winsock can take some time (because we
	// get the default Finger port using blocking calls), put
	// up an hourglass cursor so the user knows we're working on
	// something and not just out to lunch.
	SetCapture(g_hFrame);
	SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Make sure Winsock initialized and get the event dispatch
	// mechanism up and running.
	if (!InitWinsock(g_hFrame, hInstance, hPrevInstance)) {
		ReleaseCapture();
		return 0;
	}

    ReleaseCapture();

	// Show the frame window and cause it to be repainted.
	ShowWindow(g_hFrame, nCmdShow);
	UpdateWindow(g_hFrame);

	// Do the message loop.
	MSG msg;
	while (GetMessage((LPMSG)&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	// Clean up before we exit.
	CleanupInstance(hInstance);

	return msg.wParam;
}

/*
	Function: InitError
	Description:
		Handles putting up a message box for the application if it
		cannot be initialized.
*/
void InitError(void)
{
	char szMessage[80];

	wsprintf(szMessage, "ERROR: Cannot initialize %s.",
		(LPSTR)g_szFullAppName);
	// Use NULL for hwnd because application probably doesn't have
    // its window created yet.
	MessageBox(NULL, szMessage, g_szAppName, MB_ICONSTOP | MB_OK);
}

/*
	Function: InitApp
	Description:
		Registers the application's window classes and performs
		initialization that must be done for all application
		instances.  The function returns TRUE if successful and no
		errors occur, and FALSE otherwise.
*/
BOOL InitApp(HINSTANCE hInstance)
{
	WNDCLASS wc;
	BOOL bResult;

	wc.style			= CS_HREDRAW | CS_VREDRAW;	// class style
	wc.lpfnWndProc		= FrameWndProc;				// window proc
	wc.cbClsExtra		= 0;						// no extra
	wc.cbWndExtra		= 0;						// no extra
	wc.hInstance		= hInstance;				// class owner
	wc.hIcon			= LoadIcon(hInstance, g_szAppName); // icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW); // cursor
	wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH); // bg
	wc.lpszMenuName		= g_szAppName; // menu resource name
	wc.lpszClassName	= g_szAppName; // window class name

	bResult = RegisterClass(&wc);

	return bResult;
}

/*
	Function: InitInstance
	Description:
		Performs instance specific initialization, like the creation
		of the frame window, etc.  The function returns TRUE if
		successful and no errors occur, and FALSE otherwise.
*/
BOOL InitInstance(HINSTANCE hInstance)
{
	g_hFrame = CreateWindow(
		g_szAppName,					// window class name
		g_szFullAppName,				// window title text
		WS_OVERLAPPEDWINDOW | WS_VSCROLL, // window style
		CW_USEDEFAULT, CW_USEDEFAULT,	// position = default
		CW_USEDEFAULT, CW_USEDEFAULT,	// size = default
		NULL,							// no parent window
		NULL,							// menu = window class default
		hInstance,						// window owner
		NULL);							// no additional creation data

	if (!g_hFrame)
		return FALSE;

	// Initialize Microsoft 3D controls (CTL3D)
	Ctl3dRegister(hInstance);
	Ctl3dAutoSubclass(hInstance);

	return TRUE;
}

/*
	Function: InitWinsock
	Description:
		Make sure Winsock is up and running and that the Winsock event
		dispatch window is created.  Further, preform any actions that
		have to take place before the application window is created.
*/
BOOL InitWinsock(HWND hwnd, HINSTANCE hInstance,
	HINSTANCE hPrevInstance)
{
	// Make sure that we didn't have any problems
	// starting up Winsock.
	if (g_wsWinsock.LastError() != WSANOERROR) {
		ReportWSAError("Error: Can't startup Winsock.",
			g_wsWinsock.LastError(), NULL, hInstance);
		return FALSE;
	}

	// Create the Winsock message dispatch window.
	if (!WSCCreateEventWindow(hInstance, hPrevInstance)) {
		MessageBox(hwnd,
			"Error: Could not create the Winsock message window.",
			"Winsock Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

	// Look up the default port for Finger service.  Note that we'll
	// use a blocking call here since we're only doing this at
	// startup and won't need to have two of these going at once.
	struct servent FAR * lpBuffer  = getservbyname("finger", "tcp");
	if (lpBuffer == NULL) {
		// Potential problem.
		int iError = WSAGetLastError();
		if (iError == WSANO_DATA) {
			// No big deal, we just couldn't find an entry for Finger
			// in the services database.  Use the normal default
			// finger port.
			g_ushDefaultPort = DEFAULT_FINGER_PORT;
		}
		else {
			// Whoa!  Real problem of some sort.  Signal it.
			ReportWSAError(
				"Error looking up Finger service port number",
				iError, hwnd, hInstance);
			return FALSE;
		}
	}
	else {
		// No problem, so get the port number from the buffer
		// structure. Remember that the port is in network byte
		// order!!
		g_ushDefaultPort = ntohs(lpBuffer->s_port);
	}
	g_ushPort = g_ushDefaultPort;	// Current port is the default.

	return TRUE;
}

/*
	Function: CleanupInstance
	Description:
		Do any instance specific clean up before the application
		quits.
*/
void CleanupInstance(HINSTANCE hInstance)
{
	// Clean up Microsoft 3D controls (CTL3D)
	Ctl3dUnregister(hInstance);
}

/*********************************************************************
	WINDOW AND DIALOG PROCS
*********************************************************************/

/*
	Function: FrameWndProc
	Description:
		Window procedure for the application frame window.
*/
LRESULT CALLBACK FrameWndProc(HWND hwnd, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	HDC hdc;
	TEXTMETRIC tm;

	switch (msg) {
		case WM_CREATE:
			// Initialize the host and user strings.
			g_pszHost = new char[1];
			g_pszHost[0] = '\0';
			g_pszUser = new char[1];
			g_pszUser[0] = '\0';
			// Figure out the height of the text.
			hdc = GetDC(hwnd);
            SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
			GetTextMetrics(hdc, &tm);
			g_iLineHeight = tm.tmHeight;
			ReleaseDC(hwnd, hdc);
			return 0;
		HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
		HANDLE_MSG(hwnd, WM_KEYDOWN, OnKey);
		HANDLE_MSG(hwnd, WM_PAINT, OnPaint);
        HANDLE_MSG(hwnd, WM_SETCURSOR, OnSetCursor);
		HANDLE_MSG(hwnd, WM_SIZE, OnSize);
		HANDLE_MSG(hwnd, WM_VSCROLL, OnVScroll);
		case WM_DESTROY:
			// Delete all the finger text.
			if (g_ppszFingerLines != NULL) {
				// Delete the text of each line.
				for (int i = 0; i < g_iNumLines; i++)
					delete[] g_ppszFingerLines[i];
				// Delete the block of line pointers.
				delete[] g_ppszFingerLines;
			}
			PostQuitMessage(0);
			return 0;
		case WM_FINGERDONE:
			OnFingerDone(hwnd, wParam, lParam);
			return 0;
		default:
			return DefWindowProc(hwnd, msg, wParam, lParam);
	}
}

/*
	Function: ConnectDlgProc
	Description:
		Dialog proc for the connect dialog.
*/
BOOL CALLBACK ConnectDlgProc(HWND hwndDlg, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	int		id;
	BOOL	bTranslated;
	unsigned uPort;
	int		nLen;

	UNUSED_PARAMETER(lParam);

	switch (msg) {
		case WM_INITDIALOG:
			// Initialize dialog edit boxes with info from last time.
			SetDlgItemText(hwndDlg, IDD_HOST, g_pszHost);

			// Set the dialog box focus to the host name edit box and
			// select all the text in it so that the user can simply
			// type over the old name if so desired.
			SetFocus(GetDlgItem(hwndDlg, IDD_HOST));
			Edit_SetSel(GetDlgItem(hwndDlg, IDD_HOST), 0, -1);

			// Set port value text using SetDlgItemInt to do the
			// int->text conversion for us.
			SetDlgItemInt(hwndDlg, IDD_PORT, g_ushPort, FALSE);

			// Set the user name string.
			SetDlgItemText(hwndDlg, IDD_USER, g_pszUser);

			// Return FALSE because we set the focus to the host edit
			// box control.
			return FALSE;
		case WM_COMMAND:
			// Explode wParam and lParam into constituent parts.
			id = GET_WM_COMMAND_ID(wParam, lParam);
			switch (id) {
				case IDOK:
					// Retrieve the port number first.  Use a
					// temporary first, just in case things don't go
					// well.
					uPort = GetDlgItemInt(hwndDlg, IDD_PORT,
						(LPBOOL) &bTranslated, FALSE);
					// If things didn't go well, set the focus to the
					// port edit box, select all the text in it, and
					// beep.
					if (!bTranslated) {
						SetFocus(GetDlgItem(hwndDlg, IDD_PORT));
						Edit_SetSel(GetDlgItem(hwndDlg, IDD_PORT),
							0, -1);
						MessageBeep(MB_ICONHAND);
						return TRUE;
					}
					// Get the text length from the host edit box.
					nLen = Edit_GetTextLength(GetDlgItem(hwndDlg,
						IDD_HOST));
					// If the user has given us a blank string, set
					// the focus back to the edit box and beep.
					if (nLen == 0) {
						SetFocus(GetDlgItem(hwndDlg, IDD_HOST));
						Edit_SetSel(GetDlgItem(hwndDlg, IDD_HOST),
							0, -1);
						MessageBeep(MB_ICONHAND);
						return TRUE;
					}
					// Retrieve the host and user text and update the
					// g_ushPort variable with the temporary.
					delete[] g_pszHost;
					g_pszHost = new char[nLen + 1];
					GetDlgItemText(hwndDlg, IDD_HOST, g_pszHost,
						nLen + 1);
					nLen = Edit_GetTextLength(GetDlgItem(hwndDlg,
						IDD_USER));
					delete[] g_pszUser;
					g_pszUser = new char[nLen + 1];
					GetDlgItemText(hwndDlg, IDD_USER, g_pszUser,
                    	nLen + 1);
					g_ushPort = (u_short)uPort;
					EndDialog(hwndDlg, TRUE);
					return TRUE;
				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					return TRUE;
				case IDD_DEFAULTPORT:
					// Set the port edit text to the default.
					SetDlgItemInt(hwndDlg, IDD_PORT, g_ushDefaultPort,
						FALSE);
					return TRUE;
			} // switch (id)
	} // switch (msg)
	return FALSE;
}

/*********************************************************************
	WINDOW MESSAGE HANDLERS
*********************************************************************/

/*
	Function: OnCommand
	Description:
		Handle WM_COMMAND messages.
*/
void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	// Dispatch command
	switch (id) {
		case IDM_FILEEXIT:
			OnFileExit(hwnd);
			break;
		case IDM_FILECONNECT:
			OnNetworkConnect(hwnd);
			break;
		case IDM_HELPABOUT:
			OnHelpAbout(hwnd);
			break;
		default:
		// ...Or let DefWindowProc handle it if it's not ours
		FORWARD_WM_COMMAND(hwnd, id, hwndCtl, codeNotify,
			DefWindowProc);
	}
}

/*
	Function: OnFingerDone
	Description:
		Handle a WM_FINGERDONE message sent by the Finger object
		when it's done with a lookup.  The wParam contains the number
		of lines of data and the lParam contains a pointer to an
		array of character pointers to the actual null-terminated
		lines.
*/
void OnFingerDone(HWND hwnd, WPARAM wParam, LPARAM lParam)
{
	ASSERT(g_pFingerObj != NULL);

	// Delete the Finger object, set the cursor back to normal, and
	// enable the File|Connect menu choice once again.
	delete g_pFingerObj;
	g_pFingerObj = NULL;
	SetCursor(LoadCursor(NULL, IDC_ARROW));
	EnableMenuItem(GetMenu(hwnd), IDM_FILECONNECT, MF_ENABLED);
	// First delete the old data.
	if (g_ppszFingerLines != NULL) {
		// Delete the text of each line.
		for (int i = 0; i < g_iNumLines; i++)
			delete[] g_ppszFingerLines[i];
		// Delete the block of line pointers.
		delete[] g_ppszFingerLines;
	}
	// Make the new data active.
	g_ppszFingerLines = (char **) lParam;
	g_iNumLines = (int) wParam;
	// Update the vertical scroll bar.
	SetScrollRange(hwnd, SB_VERT, 0, g_iNumLines - 1, FALSE);
	SetScrollPos(hwnd, SB_VERT, 0, TRUE);
	// Force our window to update.
	InvalidateRect(hwnd, NULL, TRUE);
}

void OnKey(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags)
{
	UNUSED_PARAMETER(fDown);

    // Translate keystrokes to appropriate scroll messages.
	switch (vk) {
		case VK_HOME:
			SendMessage(hwnd, WM_VSCROLL, SB_TOP, 0L);
			break;
		case VK_END:
			SendMessage(hwnd, WM_VSCROLL, SB_BOTTOM, 0L);
			break;
		case VK_UP:
			SendMessage(hwnd, WM_VSCROLL, SB_LINEUP, 0L);
			break;
		case VK_DOWN:
			SendMessage(hwnd, WM_VSCROLL, SB_LINEDOWN, 0L);
			break;
		case VK_PRIOR:
			SendMessage(hwnd, WM_VSCROLL, SB_PAGEUP, 0L);
			break;
		case VK_NEXT:
			SendMessage(hwnd, WM_VSCROLL, SB_PAGEDOWN, 0L);
			break;
		default:
			FORWARD_WM_KEYDOWN(hwnd, vk, cRepeat, flags,
				DefWindowProc);
			break;
	}
}

/*
	Function: OnPaint
	Description:
		Handler for WM_PAINT messages.
*/
void OnPaint(HWND hwnd)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hwnd, &ps);
	if (g_iNumLines > 0) {
		SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
		RECT rect;
		GetClientRect(hwnd, &rect);
		int iLineY = rect.top;
		int iLineIndex = GetScrollPos(hwnd, SB_VERT);
		// Print all the text lines in the window, starting with
		// the top.
		while (iLineY <= rect.bottom && iLineIndex < g_iNumLines) {
			TabbedTextOut(hdc, rect.left, iLineY,
				g_ppszFingerLines[iLineIndex],
				lstrlen(g_ppszFingerLines[iLineIndex]),
				0, NULL, rect.left);
			iLineY += g_iLineHeight;
            iLineIndex++;
		}
	}
	EndPaint(hwnd, &ps);
}

/*
	Function: OnSetCursor
	Description:
		Set the cursor to an hour glass when it's in the client area
		and a lookup is in progress.
*/
BOOL OnSetCursor(HWND hwnd, HWND hwndCursor, UINT codeHitTest,
	UINT msg)
{
	UNUSED_PARAMETER(hwnd);
	UNUSED_PARAMETER(hwndCursor);
	UNUSED_PARAMETER(msg);

	// If a lookup is in progress and the cursor is in the client area
	// or the vertical scroll bar, change it to an hourglass.
	if (g_pFingerObj != NULL && (codeHitTest == HTCLIENT ||
		codeHitTest == HTVSCROLL)) {
		SetCursor(LoadCursor(NULL, IDC_WAIT));
		return TRUE;
	}
	else // Else pass the WM_SETCURSOR message to DefWindowProc.
		return FORWARD_WM_SETCURSOR(hwnd, hwndCursor, codeHitTest,
			msg, DefWindowProc);
}

/*
	Function: OnSize
	Description:
		Handle a WM_SIZE message.  In particular, store the
		vertical height to be used for calculating the number of lines
		to scroll when a page scroll is requested.
*/
void OnSize(HWND hwnd, UINT state, int cx, int cy)
{
	UNUSED_PARAMETER(hwnd);
	UNUSED_PARAMETER(state);
	UNUSED_PARAMETER(cx);

	// Save the client height for paging the scroll area when
    // processing WM_VSCROLL.
	g_iClientHeight = cy;
}

/*
	Description: OnVScroll
	Description:
		Handle scrolling of the Finger text in the client window.
		Note that this uses a very, very, very simple algorithm that
		repaints the entire client window each time a scroll occurs.
		This is pretty ugly and could be made more pretty using a
		spiffy algorithm that used ScrollWindow and just repainted the
		exposed portion of the client window following the scroll.
*/
void OnVScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
	UNUSED_PARAMETER(hwndCtl);

	// If we're in progress on a lookup, the scroll bar is locked out.
	if (g_pFingerObj != NULL)
		return;

	int iCurrentPos = GetScrollPos(hwnd, SB_VERT);

	switch (code) {
		case SB_TOP:
			iCurrentPos = 0;
			break;
		case SB_BOTTOM:
			iCurrentPos = g_iNumLines - 1;
            break;
		case SB_LINEUP:
			iCurrentPos--;
			break;
		case SB_LINEDOWN:
			iCurrentPos++;
			break;
		case SB_PAGEUP:
			iCurrentPos -= g_iClientHeight / g_iLineHeight;
			break;
		case SB_PAGEDOWN:
			iCurrentPos += g_iClientHeight / g_iLineHeight;
			break;
		case SB_THUMBPOSITION:
			iCurrentPos = pos;
			break;
	}
	iCurrentPos = max(0, min(iCurrentPos, g_iNumLines - 1));
	if (iCurrentPos != GetScrollPos(hwnd, SB_VERT)) {
		SetScrollPos(hwnd, SB_VERT, iCurrentPos, TRUE);
		InvalidateRect(hwnd, NULL, TRUE);
	}
}

/*********************************************************************
	WM_COMMAND HANDLERS
*********************************************************************/

/*
	Function: OnFileExit
	Description:
		Exits the application in response to a IDM_FILEEXIT message.
*/
void OnFileExit(HWND hwnd)
{
	SendMessage(hwnd, WM_CLOSE, 0, 0L);
}

/*
	Function: OnHelpAbout
	Description:
		Pops up the "About..." dialog box in response to IDM_HELPABOUT
		command messages.
*/
void OnHelpAbout(HWND hwnd)
{
	char szTitle[100];
	char szInfo[200];

	wsprintf(szTitle, "About %s", (LPSTR)g_szFullAppName);
	wsprintf(szInfo, "%s\r\nCopyright \251 1995\r\n"
		"David G. Roberts\r\n\r\nAn example program from the book\r\n"
		"'Developing for the Internet with Windows Sockets'",
		(LPSTR)g_szFullAppName);
	MessageBox(hwnd, szInfo, szTitle, MB_OK);
}

/*
	Function: OnNetworkConnect
	Description:
		Pops up a dialog box to get the user's input regarding the
		host and port to connect to.
*/
void OnNetworkConnect(HWND hwnd)
{
	FARPROC lpfnConnect = MakeProcInstance((FARPROC) ConnectDlgProc,
		g_hInstance);
	BOOL bResult = DialogBox(g_hInstance, "CONNECT", hwnd,
		(DLGPROC) lpfnConnect);
	FreeProcInstance(lpfnConnect);
	if (bResult == FALSE)
		return;
	// We're off and running.  Gray out the "Connect" menu item so
	// that the user can't select it again before we're done.
	EnableMenuItem(GetMenu(hwnd), IDM_FILECONNECT, MF_GRAYED);
	g_pFingerObj = new CFingerObj(g_pszHost, g_ushPort, g_pszUser,
		hwnd, WM_FINGERDONE, g_hInstance);
}
