/*
	File: NEWSCLI.CPP
	Description:
		A simple news client.
	Copyright 1995, David G. Roberts
*/

#include <stdlib.h>
#include <windows.h>
#include <windowsx.h>
#include "ctl3d.h"
#include "cwinsock.h"
#include "newscli.rh"
#include "myassert.h"

/*
	CONSTANTS
*/
#define DEFAULT_NNTP_PORT	119

#define FRAME_WINDOW_MENU_POS	2
#define GROUP_WINDOW_MENU_POS	2
#define ARTICLE_WINDOW_MENU_POS	3

/*
	TYPES
*/
typedef int INT;
typedef BOOL FAR * LPBOOL;

/*
	GLOBALS
*/
#include "newsclig.h"

/*
	FUNCTION PROTOTYPES
*/
void		CleanupInstance(HINSTANCE);
BOOL CALLBACK ConfigureDlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK GroupDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK FrameWndProc(HWND, UINT, WPARAM, LPARAM);
BOOL		InitWinsock(HINSTANCE, HINSTANCE);
BOOL		InitApp(HINSTANCE);
BOOL		InitInstance(HINSTANCE);
void		InitError(void);
LRESULT CALLBACK GroupWndProc(HWND, UINT, WPARAM, LPARAM);
INT WINAPI	WinMain(HINSTANCE, HINSTANCE, LPSTR, INT);
LRESULT CALLBACK ArticleWndProc(HWND, UINT, WPARAM, LPARAM);

/*
	MESSAGE HANDLER PROTOTYPES
*/
static void	OnClose(HWND hwnd);
static void	OnCommand(HWND, int, HWND, UINT);
BOOL OnCreate(HWND, CREATESTRUCT FAR*);

/*
	WM_COMMAND HANDLER PROTOTYPES
*/
static void	OnFileClose(HWND);
static void	OnFileExit(HWND);
static void OnFileOpenGroup(HWND);
static void	OnNewsConfigure(HWND);
static void OnWindowCloseAll(HWND);
static BOOL CALLBACK CloseEnumProc(HWND, LPARAM);
static void	OnHelpAbout(HWND);

/*********************************************************************
	WINMAIN
*********************************************************************/

/*
	Function: WinMain
	Description:
		Good ol' WinMain.
*/
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpszCmdParam, INT nCmdShow)
{
	UNUSED_PARAMETER(lpszCmdParam);

	// Save off our instance handle for later.
	g_hInstance = hInstance;

	// If no other instance is running, initialize the application
	// window classes.
	if (!hPrevInstance) {
		if (!InitApp(hInstance)) {
			InitError();
			return 0;
		}
	}

	// Load all our menus.
	g_hFrameMenu	= LoadMenu(hInstance, "FRAMEMENU");
	g_hGroupMenu	= LoadMenu(hInstance, "GROUPMENU");
	g_hArticleMenu	= LoadMenu(hInstance, "ARTICLEMENU");

	// Get the handle of the Window submenu for each MDI menu.
	g_hFrameWindowMenu = GetSubMenu(g_hFrameMenu,
		FRAME_WINDOW_MENU_POS);
	g_hGroupWindowMenu = GetSubMenu(g_hGroupMenu,
		GROUP_WINDOW_MENU_POS);
	g_hArticleWindowMenu = GetSubMenu(g_hArticleMenu,
		ARTICLE_WINDOW_MENU_POS);

	// Load the accellerators.
    g_hAccel = LoadAccelerators(hInstance, g_szAppName);

	// Initialize all instance specific stuff.  In particular, create
    // the frame window.
	if (!InitInstance(hInstance)) {
		InitError();
		return 0;
	}

	// Since initializing Winsock can take some time (because we
	// get the default Finger port using blocking calls), put
	// up an hourglass cursor so the user knows we're working on
	// something and not just out to lunch.
	SetCapture(g_hFrame);
	SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Make sure Winsock initialized and get the event dispatch
	// mechanism up and running.
	if (!InitWinsock(hInstance, hPrevInstance)) {
		ReleaseCapture();
		return 0;
	}

	ReleaseCapture();

	// Show the frame window and cause it to be repainted.
	ShowWindow(g_hFrame, nCmdShow);
	UpdateWindow(g_hFrame);

	// Do the message loop.
	MSG msg;
	while (GetMessage((LPMSG)&msg, NULL, 0, 0)) {
		if (!TranslateMDISysAccel(g_hClient, &msg) &&
			!TranslateAccelerator(g_hFrame, g_hAccel, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	DestroyMenu(g_hFrameMenu);
	DestroyMenu(g_hGroupMenu);
	DestroyMenu(g_hArticleMenu);

	// Clean up before we exit.
	CleanupInstance(hInstance);

	return msg.wParam;
}

/*
	Function: InitError
	Description:
		Handles putting up a message box for the application if it
		cannot be initialized.
*/
void InitError(void)
{
	char szMessage[80];

	wsprintf(szMessage, "ERROR: Cannot initialize %s.",
		(LPSTR)g_szFullAppName);
	// Use NULL for hwnd because application probably doesn't have
    // its window created yet.
	MessageBox(NULL, szMessage, g_szAppName, MB_ICONSTOP | MB_OK);
}

/*
	Function: InitApp
	Description:
		Registers the application's window classes and performs
		initialization that must be done for all application
		instances.  The function returns TRUE if successful and no
		errors occur, and FALSE otherwise.
*/
BOOL InitApp(HINSTANCE hInstance)
{
	WNDCLASS wc;

	// Register the frame window class.

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= FrameWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(hInstance, "APPICON");
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH) (COLOR_APPWORKSPACE + 1);
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= g_szFrameClass;

	if (!RegisterClass(&wc))
		return FALSE;

	// Register the group child window class.
	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= GroupWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= sizeof(void FAR *);
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(hInstance, "GROUPICON");
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH) (COLOR_WINDOW + 1);
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= g_szGroupClass;

	if (!RegisterClass(&wc))
		return FALSE;

	// Register the article viewer child window class.
	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= ArticleWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= sizeof(void FAR *);
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(hInstance, "ARTICLEICON");
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH) (COLOR_WINDOW + 1);
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= g_szArticleClass;

	if (!RegisterClass(&wc))
		return FALSE;

	return TRUE;
}

/*
	Function: InitInstance
	Description:
		Performs instance specific initialization, like the
		creation of the frame window, etc.  The function returns
		TRUE if successful and no errors occur, and FALSE otherwise.
*/
BOOL InitInstance(HINSTANCE hInstance)
{
	g_hFrame = CreateWindow(
		g_szFrameClass,					// window class name
		g_szFullAppName,				// window title text
		WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, // window style
		CW_USEDEFAULT, CW_USEDEFAULT,	// position = default
		CW_USEDEFAULT, CW_USEDEFAULT,	// size = default
		NULL,							// no parent window
		g_hFrameMenu,					// menu = window class default
		hInstance,						// window owner
		NULL);							// no additional creation data

	if (!g_hFrame)
		return FALSE;

	// Initialize Microsoft 3D controls (CTL3D)
	Ctl3dRegister(hInstance);
	Ctl3dAutoSubclass(hInstance);

	return TRUE;
}

/*
	Function: InitWinsock
	Description:
		Make sure Winsock is up and running and that the Winsock event
		dispatch window is created.  Further, preform any actions that
		have to take place before the application window is created.
*/
BOOL InitWinsock(HINSTANCE hInstance, HINSTANCE hPrevInstance)
{
	// Make sure that we didn't have any problems
	// starting up Winsock.
	if (g_wsWinsock.LastError() != WSANOERROR) {
		ReportWSAError("Error: Can't startup Winsock.",
			g_wsWinsock.LastError(), NULL, g_hInstance);
		return FALSE;
	}

	// Create the Winsock message dispatch window.
	if (!WSCCreateEventWindow(hInstance, hPrevInstance)) {
		MessageBox(g_hFrame,
			"Error: Could not create the Winsock message window.",
			"Winsock Error", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

	return TRUE;
}

/*
	Function: CleanupInstance
	Description:
		Do any instance specific clean-up before the application
		quits.
*/
void CleanupInstance(HINSTANCE hInstance)
{
	// Clean up Microsoft 3D controls (CTL3D)
	Ctl3dUnregister(hInstance);
}

/*********************************************************************
	WINDOW AND DIALOG PROCS
*********************************************************************/

/*
	Function: FrameWndProc
	Description:
		Window procedure for the application frame window.
*/
LRESULT CALLBACK FrameWndProc(HWND hwnd, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	switch (msg) {
		HANDLE_MSG(hwnd, WM_CREATE, OnCreate);
		HANDLE_MSG(hwnd, WM_CLOSE, OnClose);
		HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
		case WM_DESTROY:
			// Delete the config info.
			delete[] g_pszNNTPServer;
			// Post the quit message.
			PostQuitMessage(0);
			return 0;
		default:
			return DefFrameProc(hwnd, g_hClient, msg, wParam, lParam);
	}
}

/*
	Function: ConfigureDlgProc
	Description:
		Dialog proc for the configuration dialog.
*/
BOOL CALLBACK ConfigureDlgProc(HWND hwnd, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	int		id;
	BOOL	bTranslated;
	int		nLen;			

	UNUSED_PARAMETER(lParam);

	switch (msg) {
		case WM_INITDIALOG:
			// Initialize dialog edit boxes.
			SetDlgItemText(hwnd, IDD_NNTPSERVER, g_pszNNTPServer);
			SetDlgItemInt(hwnd, IDD_NNTPPORT, g_ushNNTPPort, FALSE);
			return TRUE;
		case WM_COMMAND:
			// Explode wParam and lParam into constituent parts.
			id = GET_WM_COMMAND_ID(wParam, lParam);
			switch (id) {
				case IDOK:
					// Verify all the fields.
					if (!(VerifyDlgText(hwnd, IDD_NNTPSERVER) &&
						VerifyDlgInt(hwnd, IDD_NNTPPORT, FALSE))) {
							return TRUE;
					}
					// Now that we've verified everything, retrieve
					// it all "for real."
					// First the NNTP server...
					delete[] g_pszNNTPServer;
					nLen = Edit_GetTextLength(GetDlgItem(hwnd,
						IDD_NNTPSERVER));
					g_pszNNTPServer = new char[nLen + 1];
					GetDlgItemText(hwnd, IDD_NNTPSERVER,
						g_pszNNTPServer, nLen + 1);
					// ...Then the NNTP port.
					g_ushNNTPPort = (u_short) GetDlgItemInt(hwnd,
						IDD_NNTPPORT, (LPBOOL) &bTranslated, FALSE);
					// Now we're done.
					EndDialog(hwnd, TRUE);
					return TRUE;
				case IDCANCEL:
					EndDialog(hwnd, FALSE);
					return TRUE;
				case IDD_DEFAULT:
					SetDlgItemInt(hwnd, IDD_NNTPPORT,
						g_ushNNTPDefaultPort, FALSE);
					return TRUE;
			} // switch (id)
	} // switch (msg)
	return FALSE;
}

/*
	Function: GroupDlgProc
	Description:
		Dialog proc for the Open Group dialog box.
*/
BOOL CALLBACK GroupDlgProc(HWND hwnd, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	int		id;
	int		nLen;			

	UNUSED_PARAMETER(lParam);

	switch (msg) {
		case WM_INITDIALOG:
        	// Set the keyboard focus to the group edit control.
        	SetFocus(GetDlgItem(hwnd, IDD_GROUP));
			return FALSE;
		case WM_COMMAND:
			// Explode wParam and lParam into constituent parts.
			id = GET_WM_COMMAND_ID(wParam, lParam);
			switch (id) {
				case IDOK:
					// Verify the group field.
					if (!(VerifyDlgText(hwnd, IDD_GROUP))) {
							return TRUE;
					}
					nLen = Edit_GetTextLength(GetDlgItem(hwnd,
						IDD_GROUP));
					g_pszGroup = new char[nLen + 1];
					GetDlgItemText(hwnd, IDD_GROUP,
						g_pszGroup, nLen + 1);
					// Now we're done.
					EndDialog(hwnd, TRUE);
					return TRUE;
				case IDCANCEL:
					EndDialog(hwnd, FALSE);
					return TRUE;
			} // switch (id)
	} // switch (msg)
	return FALSE;
}

/*********************************************************************
	WINDOW MESSAGE HANDLERS
*********************************************************************/

/*
	Function: OnClose
	Description:
		Handles a WM_CLOSE message to the frame window.
*/
void OnClose(HWND hwnd)
{
	// Send ourselves a Close All menu command message to close up
	// the children.
	SendMessage(hwnd, WM_COMMAND, IDM_WINDOWCLOSEALL, 0);
	if (GetWindow(g_hClient, GW_CHILD) == NULL) {
		// If the client window has no children, then all the children
		// are closed and we can continue.  Call the default window
		// proc to have it post a destroy message.
		FORWARD_WM_CLOSE(hwnd, ForwardDefFrameProc);
	}
	// One of the client's children didn't close, so simply return.
    // The user can't exit.
}

/*
	Function: OnCommand
	Description:
		Handle WM_COMMAND messages.
*/
void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	// Dispatch command
	switch (id) {
		case IDM_FILECLOSE:
			OnFileClose(hwnd);
			break;
		case IDM_FILEEXIT:
			OnFileExit(hwnd);
			break;
		case IDM_FILEOPENGROUP:
			OnFileOpenGroup(hwnd);
			break;
		case IDM_NEWSCONFIGURE:
			OnNewsConfigure(hwnd);
			break;
		case IDM_WINDOWCASCADE:
			SendMessage(g_hClient, WM_MDICASCADE, 0, 0);
			break;
		case IDM_WINDOWARRANGE:
			SendMessage(g_hClient, WM_MDIICONARRANGE, 0, 0);
			break;
		case IDM_WINDOWTILE:
			SendMessage(g_hClient, WM_MDITILE, 0, 0);
            break;
		case IDM_WINDOWCLOSEALL:
			OnWindowCloseAll(hwnd);
			break;
		case IDM_HELPABOUT:
			OnHelpAbout(hwnd);
			break;
		default:
			// ...Or send it to the active child window and see if it
			// processes it.
			HWND hwndChild = GetMDIActiveChild(g_hClient);
			if (IsWindow(hwndChild)) {
				FORWARD_WM_COMMAND(hwndChild, id, hwndCtl, codeNotify,
					SendMessage);
			}
            break;
	}
    // Always forward it to DefFrameProc after we're done.
	FORWARD_WM_COMMAND(hwnd, id, hwndCtl, codeNotify,
		ForwardDefFrameProc);
}

/*
	Function: OnCreate
	Description:
		Handle creation of the main frame window.  Creates the MDI
		client window to manage MDI children and initializes the news
		configuration information.
*/
BOOL OnCreate(HWND hwnd, CREATESTRUCT FAR* lpCreateStruct)
{
	UNUSED_PARAMETER(lpCreateStruct);

	CLIENTCREATESTRUCT ClientCreate;

	// Create the MDI client window.
	ClientCreate.hWindowMenu = g_hFrameWindowMenu;
	ClientCreate.idFirstChild = IDM_FIRSTCHILD;
	g_hClient = CreateWindow(
		"MDICLIENT",			// window class name
		NULL,					// no title for the client window
		WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE, // window style
		0, 0,					// position at 0,0
		0, 0,					// no size
		hwnd,					// parent is the frame window
		(HMENU) 1,				// Not really a menu. A child
								// window number when doing MDI.
		g_hInstance,			// owning instance
		(LPSTR)&ClientCreate);	// additional creation data

	// Set up the default port.
	g_ushNNTPDefaultPort = DEFAULT_NNTP_PORT;

	// Initialize the configuration information.
	const int STR_BASE = 10;
	const int STR_GROW_QUANTUM = 10;
	int iStrLen;
	int iReadLen;

	// Assume we're going to do everything right.
    g_bConfigured = TRUE;

    // Get the NNTP server name.
	iStrLen = STR_BASE;
	g_pszNNTPServer = new char[1];
	do {
		delete[] g_pszNNTPServer;
		iStrLen += STR_GROW_QUANTUM;
		g_pszNNTPServer = new char[iStrLen];
		iReadLen = GetPrivateProfileString("NNTP", "Server", "",
			g_pszNNTPServer, iStrLen, g_szIniFile);
	} while (iReadLen >= iStrLen - 1);
	if (lstrcmp(g_pszNNTPServer, "") == 0) {
		// If a null string got in the file or there was no entry,
        // then we still aren't configured.
		g_bConfigured = FALSE;
	}

	// Get the NNTP port number.
	g_ushNNTPPort = (u_short) GetPrivateProfileInt("NNTP", "Port", 0,
		g_szIniFile);
	if (g_ushNNTPPort == 0) {
		g_ushNNTPPort = g_ushNNTPDefaultPort;
		g_bConfigured = FALSE;
	}


    return TRUE;
}

/*********************************************************************
	WM_COMMAND HANDLERS
*********************************************************************/

/*
	Function: OnFileClose
	Description:
		Close the currently active MDI child window.
*/
void OnFileClose(HWND hwnd)
{
	UNUSED_PARAMETER(hwnd);

	HWND hChild = GetMDIActiveChild(g_hClient);

	if (hChild == NULL)
    	return;

	if (SendMessage(hChild, WM_QUERYENDSESSION, 0, 0)) {
		SendMessage(g_hClient, WM_MDIDESTROY, (WPARAM) hChild, 0);
	}
}

/*
	Function: OnFileExit
	Description:
		Exits the application in response to a IDM_FILEEXIT message.
*/
void OnFileExit(HWND hwnd)
{
	SendMessage(hwnd, WM_CLOSE, 0, 0L);
}

/*
	Function: OnFileOpenGroup
	Description:
		Create a new group window.
*/
void OnFileOpenGroup(HWND hwnd)
{
	// Open a dialog box to get the group name.
	FARPROC lpfnGroup =
		MakeProcInstance((FARPROC) GroupDlgProc, g_hInstance);
	BOOL bResult = DialogBox(g_hInstance, "GROUP", hwnd,
		(DLGPROC) lpfnGroup);
	FreeProcInstance(lpfnGroup);

    // Abort if the user hit Cancel.
	if (bResult == FALSE)
		return;

	// Create the child window.
	MDICREATESTRUCT mdicreate;

	mdicreate.szClass	= g_szGroupClass;
	mdicreate.szTitle	= g_pszGroup;
	mdicreate.hOwner	= g_hInstance;
	mdicreate.x			= CW_USEDEFAULT;
	mdicreate.y			= CW_USEDEFAULT;
	mdicreate.cx		= CW_USEDEFAULT;
	mdicreate.cy		= CW_USEDEFAULT;
	mdicreate.style		= 0;
	mdicreate.lParam	= NULL;

	// Send a message to the client window to create the new child.
	SendMessage(g_hClient, WM_MDICREATE, 0,
		(LPARAM) (LPMDICREATESTRUCT) &mdicreate);

	delete[] g_pszGroup;
}

/*
	Function: OnNewsConfigure
	Description:
		Pops up a dialog box to configure all the server name and port
		information.
*/
void OnNewsConfigure(HWND hwnd)
{
	// Put up the dialog box.
	FARPROC lpfnConfigure =
		MakeProcInstance((FARPROC) ConfigureDlgProc, g_hInstance);
	BOOL bResult = DialogBox(g_hInstance, "CONFIGURE", hwnd,
		(DLGPROC) lpfnConfigure);
	FreeProcInstance(lpfnConfigure);

	// We're only configured if we were configured before or we just
	// got configured.  If we weren't before and the user hit cancel,
	// we still aren't.
	g_bConfigured = g_bConfigured || bResult;

	// Write out the .INI file.
	char szPort[20];
	if (bResult) { // Only do this if the user hit OK.
		WritePrivateProfileString("NNTP", "Server", g_pszNNTPServer,
			g_szIniFile);
		wsprintf(szPort, "%u", g_ushNNTPPort);
		WritePrivateProfileString("NNTP", "Port", szPort,
			g_szIniFile);
	}
}

/*
	Function: OnWindowCloseAll
	Description:
		Attempts to close all the child windows.
*/
void OnWindowCloseAll(HWND hwnd)
{
	UNUSED_PARAMETER(hwnd);

	// Simply enumerate all the child windows, calling CloseEnumProc
    // for each one.
	FARPROC lpfnCloseEnum = MakeProcInstance((FARPROC) CloseEnumProc,
		g_hInstance);
	EnumChildWindows(g_hClient, (WNDENUMPROC) lpfnCloseEnum, 0);
    FreeProcInstance(lpfnCloseEnum);
}

/*
	Function: CloseEnumProc
	Description:
		The enumeration function used in the OnWindowCloseAll
		function.
*/
BOOL CALLBACK CloseEnumProc(HWND hwnd, LPARAM lParam)
{
	UNUSED_PARAMETER(lParam);

	// EnumChildWindows picks up the title windows of iconized child
	// windows.  Skip over these icon titles by checking to see if
	// they are owned by anybody.  Icon titles are owned while child
	// windows aren't.
	if (GetWindow(hwnd, GW_OWNER) != NULL) {
		return TRUE;
	}
	// Restore the child window to normal size.
	SendMessage(GetParent(hwnd), WM_MDIRESTORE, (WPARAM) hwnd, 0);
	// Ask the child whether we can close it.
	if (!SendMessage(hwnd, WM_QUERYENDSESSION, 0, 0)) {
		// If not, move on.
		return TRUE;
	}
	// Close it!
	SendMessage(GetParent(hwnd), WM_MDIDESTROY, (WPARAM) hwnd, 0);
	// Go to the next child window.
	return TRUE;
}

/*
	Function: OnHelpAbout
	Description:
		Pops up the "About..." dialog box in response to IDM_HELPABOUT
		command messages.
*/
void OnHelpAbout(HWND hwnd)
{
	char szTitle[100];
	char szInfo[200];

	wsprintf(szTitle, "About %s", (LPSTR)g_szFullAppName);
	wsprintf(szInfo, "%s\r\nCopyright \251 1995\r\n"
		"David G. Roberts\r\n\r\nAn example program from the book\r\n"
		"'Developing for the Internet with Windows Sockets'",
		(LPSTR)g_szFullAppName);
	MessageBox(hwnd, szInfo, szTitle, MB_OK);
}

/*********************************************************************
	UTILITY FUNCITONS
*********************************************************************/

/*
	Function: ForwardDefFrameProc
	Description:
		A short routine that provides some syntactic sugar to allow
		the windowsx.h FORWARD_WM_COMMAND macro to work with
		DefFrameProc.  Normally, FORWARD_WM_COMMAND packages its
		parameters and tries to call a function with the normal number
		and types of WndProc parameters (HWND, UINT, WPARAM, LPARAM).
		DefFrameProc requires the client HWND to also be present.
		Use FOWARD_WM_COMMAND to call this procedure which then adds
		the client HWND and calls DefFrameProc.
*/
void ForwardDefFrameProc(HWND hwnd, UINT msg, WPARAM wParam,
	LPARAM lParam)
{
	DefFrameProc(hwnd, g_hClient, msg, wParam, lParam);
}

/*
	Function: GetMDIActiveChild
	Description:
		This is one function that differs between Win16 and Win32.
		GetMDIActiveChild isolates those differences in this function
		rather than writing Win16/Win32 specific code everywhere.
*/
HWND GetMDIActiveChild(HWND hwndClient)
{
#ifdef WIN32
	return (HWND) SendMessage(hwndClient, WM_MDIGETACTIVE, 0, 0);
#else
	return (HWND) LOWORD(SendMessage(hwndClient, WM_MDIGETACTIVE,
		0, 0));
#endif
}

/*
	Function: VerifyDlgInt
	Description:
		Verifies that an edit field in a dialog box contains a valid
		integer.  The function works by calling GetDlgItemInt on the
		edit control and checking the bTranslated argument.  The
		caller may specify a signed or unsigned translation with the
		bSigned function argument.  If the item doesn't translate,
		the function highlights the text in the edit box, beeps, and
		returns FALSE.
*/
BOOL VerifyDlgInt(HWND hwnd, int iDlgItem, BOOL bSigned)
{
	BOOL bTranslated;

	// Try to retrieve the number.
	GetDlgItemInt(hwnd, iDlgItem, (LPBOOL) &bTranslated,
		bSigned);
	// If things didn't go well, set the focus to the
	// edit box, select all the text in it, and beep.
	if (!bTranslated) {
		SetEditErrorFocus(hwnd, iDlgItem);
		return FALSE;
	}
    return TRUE;
}

/*
	Function: VerifyDlgText
	Description:
		Verfies that an edit text control in a dialog box contains
		some text (i.e., is not null).  If the length of the text in
		the control is zero, the function sets the focus to the edit
		text item, beeps, and returns FALSE.
*/
BOOL VerifyDlgText(HWND hwnd, int iDlgItem)
{
	// Get the text length from the edit box.
	int nLen = Edit_GetTextLength(GetDlgItem(hwnd, iDlgItem));
	// If the user has given us a blank string, set the
	// focus back to the edit box and beep.
	if (nLen == 0) {
		SetEditErrorFocus(hwnd, iDlgItem);
		return FALSE;
	}
    return TRUE;
}

/*
	Function: SetEditErrorFocus
	Description:
		Sets the focus to a given dialog control, selects all its
		text, and beeps.  The dialog window handle is specified by
        hwnd and the id of the dialog item by iDlgItem.
*/
void SetEditErrorFocus(HWND hwnd, int iDlgItem)
{
	SetFocus(GetDlgItem(hwnd, iDlgItem));
	Edit_SetSel(GetDlgItem(hwnd, iDlgItem), 0, -1);
	MessageBeep(MB_ICONSTOP);
}

